function fast_1_diode_resget(Voc, Isc, Vmp,  Imp, Ns, Kv, Ki)
% -----------------------------------------------------------------------
% Funcin "fast_1_diode_resget":
%     Funcin encargada de la obtencin de las resistencias serie y
% paralelo correspondientes al modelo de 1 diodo. Difiere ligeramente
% del diagrama de flujo descrito en la memoria (resistencia paralelo
% de comienzo y condicin de detencin de bsqueda iterativa).
% -----------------------------------------------------------------------

% Definicin de constantes y asignacin de variables
Tref = 298;
Gref = 1000;
T = Tref;
G = Gref;

% Aproximacin iinicial de Rs y Rp
prompt = 'Aproximacin inicial de Rs:';
dlg_title = 'Initial Approx.';
num_lines = 1;
defaultans = {'0'};
Rs = inputdlg(prompt,dlg_title,num_lines,defaultans);
Rs = str2double(Rs{1});
Rp = inf;

% Asignamos todas las constantes y variables y las convertivos a objetos
% "tiemseries", compatibles para la lectura del espacio de trabajo de
% SIMULINK
Voc_in = timeseries(Voc);
Isc_in = timeseries(Isc);
Ns_in = timeseries(Ns);
Kv_in = timeseries(Kv);
Ki_in = timeseries(Ki);
Rs_in = timeseries(Rs);
Rp_in = timeseries(Rp);
T_in = timeseries(T);
G_in = timeseries(G);

% Ejecucin preliminar
options = simset('SrcWorkspace','current');
sim('sim_1_diode', [0, Voc], options);

% Obtencin de datos preliminares
I = I_out.Data;
V = V_out.Data;
Io = Io_out.Data(1);
Vt = Vt_out.Data(1);
Ipv = Isc;

% Clculo preliminar de diferencia de potencias
Pc = V.*I;
Pmpc = max(Pc);
Pmpe = Vmp*Imp;

if Pmpc < Pmpe
    prompt = 'Sobreestimacin del valor de resistencia en serie';
    name = 'ERROR';
    warndlg(prompt, name);
else
    % Aviso de clculo
    msgbox('Calculando resistencia, puede tardar varios minutos...');

    % Clculo iterativo:
    while (Pmpc > Pmpe)
        Rs = Rs + 0.001; % Aumento de Rs por cada iteracin
        Rp = (Vmp+Imp*Rs)/(Ipv - Io*(exp((Vmp + Imp*Rs)/Vt) - 1) - Imp);
        Rs_in = timeseries(Rs);
        Rp_in = timeseries(Rp);
        sim('sim_1_diode', [0, Voc], options);
        I = I_out.Data;
        V = V_out.Data;
        Pc = V.*I;
        Pmpc = max(Pc);
    end

    % Final de clculo y ventana de informacin
    msg = {strcat('Rs:  ', num2str(Rs), ' Ohm'), ...
        strcat('Rp:  ', num2str(Rp), ' Ohm')};
    msgbox(msg);
end