function [I, V] = fast_diode(Voc, Isc, Ns, Kv, Ki, Rs, Rp, G, T, ...
    n_points, mode)
% -----------------------------------------------------------------------
% Funcin "fast_diode":
%     Funcin simple que convierte los datos proporcionados necesarios
% para el modelo en formato "timeseries" y as prepararlos para el
% espacio de trabajo de SIMULINK, herramienta con la que se ha
% implementado los modelos, los cuales toman los datos del espacio de
% MATLAB actual. Permite seleccionar el uso de los dos tipos de modelo
% de clula solar que se han desarrollado.
% -----------------------------------------------------------------------

% Convertimos los datos proporcionados a "timeseries" con los nombres
% correspondientes leidos por los modelos de simulink
Voc_in = timeseries(Voc);
Isc_in = timeseries(Isc);
Ns_in = timeseries(Ns);
Kv_in = timeseries(Kv);
Ki_in = timeseries(Ki);
G_in = timeseries(G);
T_in = timeseries(T);

% Se determinan las opciones para el modelo de SIMULINK de forma externa:
%     -Tamao mximo de paso
%     -Seleccin del espacio de MATLAB actual para la lectura de 
%     parmetros
options = simset('MaxStep', num2str(Voc/n_points), 'SrcWorkspace', ...
    'current');
set_param('sim_2_diode','AlgebraicLoopSolver','LineSearch'); % Evita 
    % problemas con la resolucin de cantidades cero o muy pequeas
% Seleccin de modo
if mode == true
    Rs_in = timeseries(Rs(2));
    Rp_in = timeseries(Rp(2));
    sim('sim_2_diode', [0, Voc], options);
else
    Rs_in = timeseries(Rs(1));
    Rp_in = timeseries(Rp(1));
    sim('sim_1_diode', [0, Voc], options);
end

% Devolucin de datos
I = I_out.Data;
V = V_out.Data;