function fused_filename = file_fuser()
% -----------------------------------------------------------------------
% Funcin "file_fuser":
%     Esta funcin se encarga de solucionar un problema de Meteonorm,
% el cual no interpola a minutos la temperatura por horas, lo que deja
% una columna de datos vaca en el modelo de datos "standard minute".
% Se soluciona interpolando los datos "standard" a minutos mediante
% interpolacin lineal, y agregando todos los datos obtenidos al fichero
% de minutos sinla columna de temperatura.
% Se implementa como una utilidad para la aplicacin central del
% simulador.
% -----------------------------------------------------------------------

% Obtencin de la ruta del fichero "standard"
[hour_filename, hour_path] = uigetfile('.dat');
if strcmp(hour_filename, '') % Si no se ha seleccionado nada
    % No se ejecuta nada, error
else
    % Obtencin del fichero de "standard minute"
    [minute_filename, minute_path] = uigetfile('.dat');
    % Creacin de la ruta y el nombre del fichero final completo
    city_name = minute_filename(1:4);
    fused_path = strcat(minute_path, city_name, 'minu_wtemp.dat');
    
    % Apertura y lectura del fichero "standard", obtencin de los datos
    % de temperatura
    fid_hour = fopen(strcat(hour_path, hour_filename), 'r' );
    formatspec = '%*d%*d%*d%*d%*d%*d%*d%*d%*d%f';
    hour_temp = cell2mat(textscan(fid_hour, formatspec));
    fclose(fid_hour);

    % Interpolacin de los datos
    minu_temp = 0;
    for i = 1:length(hour_temp)
        ini_val = hour_temp(i);
        if i == length(hour_temp);
            fin_val = ini_val - (hour_temp(i-1) - ini_val);
        else
            fin_val = hour_temp(i+1);
        end
        interval = (fin_val-ini_val)/60;
        for j = 1:60
           minu_temp((i-1)*60+j) = ini_val + interval*(j-1);
        end
    end
    clear hour_temp; % Limpiamos de memoria el vector de temp. sin interp.

    % Apertura y lectura del fichero "standard minute"
    fid_minu = fopen( strcat(minute_path, minute_filename), 'r' );
    formatspec = '%d%d%d%d%d%d%f%d%d%d%d%d';
    minu_matrix = textscan(fid_minu, formatspec);
    fclose(fid_minu);
    % Aadimos a la matriz de los datos leidos el vector de temperaturas
    % interpoladas
    minu_matrix{13} = minu_temp.';

    % Creacin de barra de progreso para la escritura del fichero
    progressbar(0)
    progressbar('Progreso de la fusin');

    % Creacin y apertura del fichero fusionado
    fid_fused = fopen(fused_path, 'wt');
    % Divisin de datos a escribir, no se pueden guardar datos de
    % distinto formato en el mismo vector, se hace en 4 tandas
    formatspec_1 = '%d\t%d\t%d\t%d\t%d\t%d\t';
    formatspec_2 = '%0.1f\t';
    formatspec_3 = '%d\t%d\t%d\t%d\t%d\t';
    formatspec_4 = '%0.2f\r\n';
    % Escritura de las filas del fichero de datos fusionado
    for i = 1:length(minu_matrix{1})
        min_vec_1 = [minu_matrix{1}(i); minu_matrix{2}(i); ...
            minu_matrix{3}(i); minu_matrix{4}(i); ...
            minu_matrix{5}(i); minu_matrix{6}(i)];
        min_vec_2 = minu_matrix{7}(i);
        min_vec_3 = [minu_matrix{8}(i); minu_matrix{9}(i); ...
            minu_matrix{10}(i); minu_matrix{11}(i); ...
            minu_matrix{12}(i)];
        min_vec_4 = minu_matrix{13}(i);
        fprintf(fid_fused, formatspec_1, min_vec_1);
        fprintf(fid_fused, formatspec_2, min_vec_2);
        fprintf(fid_fused, formatspec_3, min_vec_3);
        fprintf(fid_fused, formatspec_4, min_vec_4);
        progressbar((1/length(minu_matrix{1}))*i);
        disp(i);
    end
    % Cierre del fichero y cierre de la barra de progreso
    fclose(fid_fused);
    progressbar(1);
end