% -----------------------------------------------------------------------
% Script "model_evaluation":
%     Script usado para comparar los resultados obtenidos por los
% simuladores entre s y con los valores reales para obtener
% conclusiones sobre su funcionamiento y precisin
% -----------------------------------------------------------------------

% Limpieza previa
clear;
close all;

% Carga de modelos y limpieza de memoria
load_system('sim_1_diode'); % 1 diodo
load_system('sim_2_diode'); % 2 diodos
precision = 100000;

% Datos de mdulo
% -----------------------------------------------------------------------
Voc_stc = 46.2;
Isc_stc = 9.48;
Pmp_stc = 340;
Vmp_stc = 37.9;
Imp_stc = 8.97;
Voc_noct = 43;
Isc_noct = 7.66;
Pmp_noct = 250;
Vmp_noct = 34.9;
Imp_noct = 7.16;
Ns = 72;
Kv = -143;
Ki = 5.02;
Rs = [0.295, 0.291];
Rp = [581.4, 604.7];
TONC = 0.0288;
% -----------------------------------------------------------------------


% Datos de pruebas
% -----------------------------------------------------------------------
G_stc = 1000;
Tc_stc = 25 + 273.15;
G_noct = 800;
Tc_noct = 43 + 273.15;
G_gtest = [200, 400, 600, 800, 1000];
Tc_gtest = 25 + 273.15;
G_ttest = 1000;
Tc_ttest = [5, 25, 45, 65] + 273.15;
% -----------------------------------------------------------------------

% Barra de progreso
progressbar(0);
progressbar('Progreso simulacin');
n_sims = 22;

% Test STC
[I_stc_1, V_stc_1] = fast_diode(Voc_stc, Isc_stc, Ns, Kv, Ki, Rs, ...
    Rp, G_stc, Tc_stc, precision, false);
[I_stc_2, V_stc_2] = fast_diode(Voc_stc, Isc_stc, Ns, Kv, Ki, Rs, ...
    Rp, G_stc, Tc_stc, precision, true);
progressbar(1/11);
I_stc_sc_1_index = find(V_stc_1 <= 0);
V_stc_oc_1_index = find(I_stc_1 <= 0);
I_stc_sc_1 = I_stc_1(I_stc_sc_1_index(1));
V_stc_oc_1 = V_stc_1(V_stc_oc_1_index(1));
P_stc_1 = I_stc_1.*V_stc_1;
P_stc_mp_1 = max(P_stc_1);
I_V_stc_mp_1_index = find(P_stc_1 == P_stc_mp_1);
I_stc_mp_1 = I_stc_1(I_V_stc_mp_1_index);
V_stc_mp_1 = V_stc_1(I_V_stc_mp_1_index);
% disp(I_stc_sc_1);
% disp(V_stc_oc_1);
% disp(P_stc_mp_1);
% disp(I_stc_mp_1);
% disp(V_stc_mp_1);
I_stc_sc_2_index = find(V_stc_2 <= 0);
V_stc_oc_2_index = find(I_stc_2 <= 0);
I_stc_sc_2 = I_stc_2(I_stc_sc_2_index(1));
V_stc_oc_2 = V_stc_2(V_stc_oc_2_index(1));
P_stc_2 = I_stc_2.*V_stc_2;
P_stc_mp_2 = max(P_stc_2);
I_V_stc_mp_2_index = find(P_stc_2 == P_stc_mp_2);
I_stc_mp_2 = I_stc_2(I_V_stc_mp_2_index);
V_stc_mp_2 = V_stc_2(I_V_stc_mp_2_index);
% disp(I_stc_sc_2);
% disp(V_stc_oc_2);
% disp(P_stc_mp_2);
% disp(I_stc_mp_2);
% disp(V_stc_mp_2);

% Test NOCT
[I_noct_1, V_noct_1] = fast_diode(Voc_stc, Isc_stc, Ns, Kv, Ki, Rs, ...
    Rp, G_noct, Tc_noct, precision, false);
[I_noct_2, V_noct_2] = fast_diode(Voc_stc, Isc_stc, Ns, Kv, Ki, Rs, ...
    Rp, G_noct, Tc_noct, precision, true);
progressbar(2/11);
I_noct_sc_1_index = find(V_noct_1 <= 0);
V_noct_oc_1_index = find(I_noct_1 <= 0);
I_noct_sc_1 = I_noct_1(I_noct_sc_1_index(1));
V_noct_oc_1 = V_noct_1(V_noct_oc_1_index(1));
P_noct_1 = I_noct_1.*V_noct_1;
P_noct_mp_1 = max(P_noct_1);
I_V_noct_mp_1_index = find(P_noct_1 == P_noct_mp_1);
I_noct_mp_1 = I_noct_1(I_V_noct_mp_1_index);
V_noct_mp_1 = V_noct_1(I_V_noct_mp_1_index);
% disp(I_noct_sc_1);
% disp(V_noct_oc_1);
% disp(P_noct_mp_1);
% disp(I_noct_mp_1);
% disp(V_noct_mp_1);
I_noct_sc_2_index = find(V_noct_2 <= 0);
V_noct_oc_2_index = find(I_noct_2 <= 0);
I_noct_sc_2 = I_noct_2(I_noct_sc_2_index(1));
V_noct_oc_2 = V_noct_2(V_noct_oc_2_index(1));
P_noct_2 = I_noct_2.*V_noct_2;
P_noct_mp_2 = max(P_noct_2);
I_V_noct_mp_2_index = find(P_noct_2 == P_noct_mp_2);
I_noct_mp_2 = I_noct_2(I_V_noct_mp_2_index);
V_noct_mp_2 = V_noct_2(I_V_noct_mp_2_index);
% disp(I_noct_sc_2);
% disp(V_noct_oc_2);
% disp(P_noct_mp_2);
% disp(I_noct_mp_2);
% disp(V_noct_mp_2);

% Anlisis de puntos generados por el simulador
i_v_length = length(I_stc_1);
gtest_length = length(G_gtest);
ttest_length = length(Tc_ttest);

% Test irradiancia variable
I_gtest_1 = zeros([gtest_length, i_v_length]);
V_gtest_1 = zeros([gtest_length, i_v_length]);
I_gtest_2 = zeros([gtest_length, i_v_length]);
V_gtest_2 = zeros([gtest_length, i_v_length]);
I_gtest_sc_1 = zeros([gtest_length, 1]);
V_gtest_oc_1 = zeros([gtest_length, 1]);
P_gtest_mp_1 = zeros([gtest_length, 1]);
I_gtest_mp_1 = zeros([gtest_length, 1]);
V_gtest_mp_1 = zeros([gtest_length, 1]);
I_gtest_sc_2 = zeros([gtest_length, 1]);
V_gtest_oc_2 = zeros([gtest_length, 1]);
P_gtest_mp_2 = zeros([gtest_length, 1]);
I_gtest_mp_2 = zeros([gtest_length, 1]);
V_gtest_mp_2 = zeros([gtest_length, 1]);
for i = 1:length(G_gtest)
    [I_gtest_1(i, :), V_gtest_1(i, :)] = fast_diode(Voc_stc, Isc_stc, ...
        Ns, Kv, Ki, Rs, Rp, G_gtest(i), Tc_gtest, precision, false);
    [I_gtest_2(i, :), V_gtest_2(i, :)] = fast_diode(Voc_stc, Isc_stc, ...
        Ns, Kv, Ki, Rs, Rp, G_gtest(i), Tc_gtest, precision, true);
    progressbar((i+2)/11);
    I_gtest_sc_1_index = find(V_gtest_1(i, :) <= 0);
    V_gtest_oc_1_index = find(I_gtest_1(i, :) <= 0);
    I_gtest_sc_1(i) = I_gtest_1(i, I_gtest_sc_1_index(1));
    V_gtest_oc_1(i) = V_gtest_1(i, V_gtest_oc_1_index(1));
    P_gtest_1 = I_gtest_1(i, :).*V_gtest_1(i, :);
    P_gtest_mp_1(i) = max(P_gtest_1);
    I_V_gtest_mp_1_index = find(P_gtest_1 == P_gtest_mp_1(i));
    I_gtest_mp_1(i) = I_gtest_1(i, I_V_gtest_mp_1_index);
    V_gtest_mp_1(i) = V_gtest_1(i, I_V_gtest_mp_1_index);
%     disp(I_gtest_sc_1(i));
%     disp(V_gtest_oc_1(i));
%     disp(P_gtest_mp_1(i));
%     disp(I_gtest_mp_1(i));
%     disp(V_gtest_mp_1(i));
    I_gtest_sc_2_index = find(V_gtest_2(i, :) <= 0);
    V_gtest_oc_2_index = find(I_gtest_2(i, :) <= 0);
    I_gtest_sc_2(i) = I_gtest_2(i, I_gtest_sc_2_index(1));
    V_gtest_oc_2(i) = V_gtest_2(i, V_gtest_oc_2_index(1));
    P_gtest_2 = I_gtest_2(i, :).*V_gtest_2(i, :);
    P_gtest_mp_2(i) = max(P_gtest_2);
    I_V_gtest_mp_2_index = find(P_gtest_2 == P_gtest_mp_2(i));
    I_gtest_mp_2(i) = I_gtest_2(i, I_V_gtest_mp_2_index);
    V_gtest_mp_2(i) = V_gtest_2(i, I_V_gtest_mp_2_index);
%     disp(I_gtest_sc_2(i));
%     disp(V_gtest_oc_2(i));
%     disp(P_gtest_mp_2(i));
%     disp(I_gtest_mp_2(i));
%     disp(V_gtest_mp_2(i));
end

% Test temperatura variable
I_ttest_1 = zeros([ttest_length, i_v_length]);
V_ttest_1 = zeros([ttest_length, i_v_length]);
I_ttest_2 = zeros([ttest_length, i_v_length]);
V_ttest_2 = zeros([ttest_length, i_v_length]);
I_ttest_sc_1 = zeros([ttest_length, 1]);
V_ttest_oc_1 = zeros([ttest_length, 1]);
P_ttest_mp_1 = zeros([ttest_length, 1]);
I_ttest_mp_1 = zeros([ttest_length, 1]);
V_ttest_mp_1 = zeros([ttest_length, 1]);
I_ttest_sc_2 = zeros([ttest_length, 1]);
V_ttest_oc_2 = zeros([ttest_length, 1]);
P_ttest_mp_2 = zeros([ttest_length, 1]);
I_ttest_mp_2 = zeros([ttest_length, 1]);
V_ttest_mp_2 = zeros([ttest_length, 1]);
for i = 1:length(Tc_ttest)
    [I_ttest_1(i, :), V_ttest_1(i, :)] = fast_diode(Voc_stc, Isc_stc, ...
        Ns, Kv, Ki, Rs, Rp, G_ttest, Tc_ttest(i), precision, false);
    [I_ttest_2(i, :), V_ttest_2(i, :)] = fast_diode(Voc_stc, Isc_stc, ...
    Ns, Kv, Ki, Rs, Rp, G_ttest, Tc_ttest(i), precision, true);
    progressbar((i+7)/11);
    I_ttest_sc_1_index = find(V_ttest_1(i, :) <= 0);
    V_ttest_oc_1_index = find(I_ttest_1(i, :) <= 0);
    % Fix----------------------------------------------------------------
    if isempty(V_ttest_oc_1_index)
        V_ttest_oc_1_index(1) = length(I_ttest_1(i, :));
    end
    % Fix----------------------------------------------------------------
    I_ttest_sc_1(i) = I_ttest_1(i, I_ttest_sc_1_index(1));
    V_ttest_oc_1(i) = V_ttest_1(i, V_ttest_oc_1_index(1));
    P_ttest_1 = I_ttest_1(i, :).*V_ttest_1(i, :);
    P_ttest_mp_1(i) = max(P_ttest_1);
    I_V_ttest_mp_1_index = find(P_ttest_1 == P_ttest_mp_1(i));
    I_ttest_mp_1(i) = I_ttest_1(i, I_V_ttest_mp_1_index);
    V_ttest_mp_1(i) = V_ttest_1(i, I_V_ttest_mp_1_index);
%     disp(I_ttest_sc_1(i));
%     disp(V_ttest_oc_1(i));
%     disp(P_ttest_mp_1(i));
%     disp(I_ttest_mp_1(i));
%     disp(V_ttest_mp_1(i));
    I_ttest_sc_2_index = find(V_ttest_2(i, :) <= 0);
    V_ttest_oc_2_index = find(I_ttest_2(i, :) <= 0);
    % Fix----------------------------------------------------------------
    if isempty(V_ttest_oc_2_index)
        V_ttest_oc_2_index(1) = length(I_ttest_2(i, :));
    end
    % Fix----------------------------------------------------------------
    I_ttest_sc_2(i) = I_ttest_2(i, I_ttest_sc_2_index(1));
    V_ttest_oc_2(i) = V_ttest_2(i, V_ttest_oc_2_index(1));
    P_ttest_2 = I_ttest_2(i, :).*V_ttest_2(i, :);
    P_ttest_mp_2(i) = max(P_ttest_2);
    I_V_ttest_mp_2_index = find(P_ttest_2 == P_ttest_mp_2(i));
    I_ttest_mp_2(i) = I_ttest_2(i, I_V_ttest_mp_2_index);
    V_ttest_mp_2(i) = V_ttest_2(i, I_V_ttest_mp_2_index);
%     disp(I_ttest_sc_2(i));
%     disp(V_ttest_oc_2(i));
%     disp(P_ttest_mp_2(i));
%     disp(I_ttest_mp_2(i));
%     disp(V_ttest_mp_2(i));
end

% Finalizacin de simulaciones
progressbar(1);


% Comparaciones Grficas
C = {'k','b','r','g','y'};

% Figura 1: STC
figure(1);
hold on;
plot(V_stc_1, I_stc_1, 'color', C{1}, 'LineWidth', 2);
xlabel('V');
ylabel('I');
xlim([0 inf]);
ylim([0 inf]);
plot(V_stc_2, I_stc_2, 'color', C{2}, 'LineWidth', 2);
xlabel('V');
ylabel('I');
xlim([0 inf]);
ylim([0 inf]);
legend('1 diodo', '2 diodos');
hold off;

% Figura 2: NOCT
figure(2);
hold on;
plot(V_noct_1, I_noct_1, 'color', C{1}, 'LineWidth', 2);
xlabel('V');
ylabel('I');
xlim([0 inf]);
ylim([0 inf]);
plot(V_noct_2, I_noct_2, 'color', C{2}, 'LineWidth', 2);
xlabel('V');
ylabel('I');
xlim([0 inf]);
ylim([0 inf]);
legend('1 diodo', '2 diodos');
hold off;

% Figura 3 y 4: Comparativa irradiancias
% Modelo de 1 diodo
figure(3);
hold on;
for i = 1:length(G_gtest)
    plot(V_gtest_1(i, :), I_gtest_1(i, :), 'color', C{i}, ...
        'LineWidth', 2);
    xlabel('V')
    ylabel('I')
    xlim([0 inf]) 
    ylim([0 inf])
end
legend('200 W/m^2', '400 W/m^2', '600 W/m^2', '800 W/m^2', '1000 W/m^2');
hold off;
% Modelo de 2 diodos
figure(4);
hold on;
for i = 1:length(G_gtest)
    plot(V_gtest_2(i, :), I_gtest_2(i, :), 'color', C{i}, ...
        'LineWidth', 2);
    xlabel('V')
    ylabel('I')
    xlim([0 inf]) 
    ylim([0 inf])
end
legend('200 W/m^2', '400 W/m^2', '600 W/m^2', '800 W/m^2', '1000 W/m^2');
hold off;

% Figura 5 y 6: Comparativa irradiancias
% Modelo de 1 diodo
figure(5);
hold on;
for i = 1:length(Tc_ttest)
    plot(V_ttest_1(i, :), I_ttest_1(i, :), 'color', C{i}, ...
        'LineWidth', 2);
    xlabel('V')
    ylabel('I')
    xlim([0 inf]) 
    ylim([0 inf])
end
legend('5 C', '25 C', '45 C', '65 C');
hold off;
% Modelo de 2 diodos
figure(6);
hold on;
for i = 1:length(Tc_ttest)
    plot(V_ttest_2(i, :), I_ttest_2(i, :), 'color', C{i}, ...
        'LineWidth', 2);
    xlabel('V')
    ylabel('I')
    xlim([0 inf]) 
    ylim([0 inf])
end
legend('5 C', '25 C', '45 C', '65 C');
hold off;

% Escritura con resultados de precisin
file_path = uigetdir(pwd);
filename = strcat(file_path, '\model_evaluation_results.txt');

fid = fopen(filename, 'wt');
fprintf(fid, 'Resultados STC\r\n');
fprintf(fid, '\tModelo de 1 diodo:\r\n');
error = abs(((Voc_stc-V_stc_oc_1)/Voc_stc)*100);
fprintf(fid, '\t\tVoc: %f%% de error\r\n', error);
error = abs(((Isc_stc-I_stc_sc_1)/Isc_stc)*100);
fprintf(fid, '\t\tIsc: %f%% de error\r\n', error);
error = abs(((Pmp_stc-P_stc_mp_1)/Pmp_stc)*100);
fprintf(fid, '\t\tPmp: %f%% de error\r\n', error);
error = abs(((Vmp_stc-V_stc_mp_1)/Vmp_stc)*100);
fprintf(fid, '\t\tVmp: %f%% de error\r\n', error);
error = abs(((Imp_stc-I_stc_mp_1)/Imp_stc)*100);
fprintf(fid, '\t\tImp: %f%% de error\r\n', error);
fprintf(fid, '\tModelo de 2 diodos:\r\n');
error = abs(((Voc_stc-V_stc_oc_2)/Voc_stc)*100);
fprintf(fid, '\t\tVoc: %f%% de error\r\n', error);
error = abs(((Isc_stc-I_stc_sc_2)/Isc_stc)*100);
fprintf(fid, '\t\tIsc: %f%% de error\r\n', error);
error = abs(((Pmp_stc-P_stc_mp_2)/Pmp_stc)*100);
fprintf(fid, '\t\tPmp: %f%% de error\r\n', error);
error = abs(((Vmp_stc-V_stc_mp_2)/Vmp_stc)*100);
fprintf(fid, '\t\tVmp: %f%% de error\r\n', error);
error = abs(((Imp_stc-I_stc_mp_2)/Imp_stc)*100);
fprintf(fid, '\t\tImp: %f%% de error\r\n', error);
fprintf(fid, '\r\nResultados NOCT\r\n');
fprintf(fid, '\tModelo de 1 diodo:\r\n');
error = abs(((Voc_noct-V_noct_oc_1)/Voc_noct)*100);
fprintf(fid, '\t\tVoc: %f%% de error\r\n', error);
error = abs(((Isc_noct-I_noct_sc_1)/Isc_noct)*100);
fprintf(fid, '\t\tIsc: %f%% de error\r\n', error);
error = abs(((Pmp_noct-P_noct_mp_1)/Pmp_noct)*100);
fprintf(fid, '\t\tPmp: %f%% de error\r\n', error);
error = abs(((Vmp_noct-V_noct_mp_1)/Vmp_noct)*100);
fprintf(fid, '\t\tVmp: %f%% de error\r\n', error);
error = abs(((Imp_noct-I_noct_mp_1)/Imp_noct)*100);
fprintf(fid, '\t\tImp: %f%% de error\r\n', error);
fprintf(fid, '\tModelo de 2 diodos:\r\n');
error = abs(((Voc_noct-V_noct_oc_2)/Voc_noct)*100);
fprintf(fid, '\t\tVoc: %f%% de error\r\n', error);
error = abs(((Isc_noct-I_noct_sc_2)/Isc_noct)*100);
fprintf(fid, '\t\tIsc: %f%% de error\r\n', error);
error = abs(((Pmp_noct-P_noct_mp_2)/Pmp_noct)*100);
fprintf(fid, '\t\tPmp: %f%% de error\r\n', error);
error = abs(((Vmp_noct-V_noct_mp_2)/Vmp_noct)*100);
fprintf(fid, '\t\tVmp: %f%% de error\r\n', error);
error = abs(((Imp_noct-I_noct_mp_2)/Imp_noct)*100);
fprintf(fid, '\t\tImp: %f%% de error\r\n', error);
fprintf(fid, '\r\n\r\n----COMPARATIVAS (respecto modelo de 2 diodos)----\r\n');
fprintf(fid, '\r\nResultados comparativa en irradiancia\r\n');
for i = 1:length(G_gtest)
    fprintf(fid, '\tPara %d W/m^2:\r\n', G_gtest(i));
    error = abs(((V_gtest_oc_1(i)-V_gtest_oc_2(i))/V_gtest_oc_2(i))*100);
    fprintf(fid, '\t\tVoc: %f%% de error\r\n', error);
    error = abs(((I_gtest_sc_1(i)-I_gtest_sc_2(i))/I_gtest_sc_2(i))*100);
    fprintf(fid, '\t\tIsc: %f%% de error\r\n', error);
    error = abs(((P_gtest_mp_1(i)-P_gtest_mp_2(i))/P_gtest_mp_2(i))*100);
    fprintf(fid, '\t\tPmp: %f%% de error\r\n', error);
    error = abs(((V_gtest_mp_1(i)-V_gtest_mp_2(i))/V_gtest_mp_2(i))*100);
    fprintf(fid, '\t\tVmp: %f%% de error\r\n', error);
    error = abs(((I_gtest_mp_1(i)-I_gtest_mp_2(i))/I_gtest_mp_2(i))*100);
    fprintf(fid, '\t\tImp: %f%% de error\r\n', error);
end
fprintf(fid, '\r\nResultados comparativa en temperatura:\r\n');
for i = 1:length(Tc_ttest)
    fprintf(fid, '\tPara %d C: \r\n', Tc_ttest(i)-273.15);
    error = abs(((V_ttest_oc_1(i)-V_ttest_oc_2(i))/V_ttest_oc_2(i))*100);
    fprintf(fid, '\t\tVoc: %f%% de error\r\n', error);
    error = abs(((I_ttest_sc_1(i)-I_ttest_sc_2(i))/I_ttest_sc_2(i))*100);
    fprintf(fid, '\t\tIsc: %f%% de error\r\n', error);
    error = abs(((P_ttest_mp_1(i)-P_ttest_mp_2(i))/P_ttest_mp_2(i))*100);
    fprintf(fid, '\t\tPmp: %f%% de error\r\n', error);
    error = abs(((V_ttest_mp_1(i)-V_ttest_mp_2(i))/V_ttest_mp_2(i))*100);
    fprintf(fid, '\t\tVmp: %f%% de error\r\n', error);
    error = abs(((I_ttest_mp_1(i)-I_ttest_mp_2(i))/I_ttest_mp_2(i))*100);
    fprintf(fid, '\t\tImp: %f%% de error\r\n', error);
end

fclose(fid);
