function [model_name, rend, Voc, Isc, Pmp, Imp, Vmp, Ns, Kv, Ki, Rs, ...
    Rp, TONC, surface] = read_modulefile(filename)
% -----------------------------------------------------------------------
% Funcin "read_modulefile":
%     Funcin encargada de la lectura de los parmetros correspondientes
% a un mdulo solar determinado. Los parmetros se almacenan en un
% archivo de extensin .csv: en la primera columna se nombran los
% parmetros almacenados y en la segunda los valores de dichos
% parmetros. Los datos correspondientes a un fichero de mdulo son:
%     -Fabricante e indentificador del mdulo
%     -Tensin de circuito abierto (Voc) [Voltios, V]
%     -Corriente de cortocircuito (Isc) [Amperios, A]
%     -Potencia mxima (Pmp) [Vatios, W]
%     -Corriente de pot. mxima (Imp) [Amperios, A]
%     -Tensin de pot. mxima (Vmp) [Voltios, V]
%     -Nmero de clulas por mdulo (Ns)
%     -Coeficiente de temp. respecto a Voc (Kv) [mV/C]
%     -Coeficiente de temp. respecto a Isc (Ki) [mA/C]
%     -Resistencias en serie para mod. de 1 y 2 diodos (Rs) [Ohmios]
%     -Resistencias en paralelo para mod. de 1 y 2 diodos (Rp) [Ohmios]
%     -Temperatura de operacin nominal de la clula (TONC) [C*m/W]
%     -Superficie total del panel [m^2]
% -----------------------------------------------------------------------

% Apertura y lectura del fichero, ignorando la primera fila
fid = fopen( filename, 'r' );
formatspec = '%s%f';
values = textscan(fid, formatspec, 'Delimiter', ',', ...
    'Headerlines', 1);
fclose(fid);

% Asignacin de los valores leidos a las variables devueltas por la
% funcin
model_name = values{1, 1}{1}; % Nombre del modelo
rend = values{1, 2}(2); % Rendimiento de la clula
Voc = values{1, 2}(3); % Voltaje de cortocircuito
Isc = values{1, 2}(4); % Corriente de cortocircuito
Pmp = values{1, 2}(5); % Potencia mxima
Imp = values{1, 2}(6); % Corriente de potencia mxima
Vmp = values{1, 2}(7); % Voltaje de potencia mxima
Ns = values{1, 2}(8); % Nmero de clulas por mdulo
Kv = values{1, 2}(9); % Coef. de temp. respecto a Voc
Ki = values{1, 2}(10); % Coef. de temp. respecto a Isc
Rs(1) = values{1, 2}(11); % Res. serie del modelo de 1 diodo
Rp(1) = values{1, 2}(12); % Res. paralelo del modelo de 1 diodo
Rs(2) = values{1, 2}(13); % Res. serie del modelo de 2 diodos
Rp(2) = values{1, 2}(14); % Res. paralelo del modelo de 2 diodos
TONC = values{1, 2}(15); % Temperatura de operacin nominal
surface = values{1, 2}(16); % Superficie total del panel
